@@ -25,12 +25,10 @@ pub struct Matrix<C> {
 impl<C: Clone> Matrix<C> {
     /// Create new matrix with an initial value.
     pub fn new(rows: usize, columns: usize, value: C) -> Self {
-        let mut v = Vec::with_capacity(rows * columns);
-        v.resize(rows * columns, value);
         Self {
             rows,
             columns,
-            data: v,
+            data: vec![value; rows * columns],
         }
     }
 
@@ -240,14 +238,12 @@ impl<C> Matrix<C> {
     /// ```
     /// use pathfinding::matrix::*;
     ///
-    /// # fn main() -> Result<(), MatrixFormatError> {
     /// let input = "abc\ndef";
     /// let matrix = Matrix::from_rows(input.lines().map(|l| l.chars()))?;
     /// assert_eq!(matrix.rows, 2);
     /// assert_eq!(matrix.columns, 3);
     /// assert_eq!(matrix[&(1, 1)], 'e');
-    /// # Ok(())
-    /// # }
+    /// # Ok::<_, MatrixFormatError>(())
     /// ```
     pub fn from_rows<IR, IC>(rows: IR) -> Result<Self, MatrixFormatError>
     where
@@ -509,9 +505,21 @@ impl<C> DerefMut for Matrix<C> {
     }
 }
 
+impl<C, IC> FromIterator<IC> for Matrix<C>
+where
+    IC: IntoIterator<Item = C>,
+{
+    fn from_iter<T: IntoIterator<Item = IC>>(iter: T) -> Self {
+        Matrix::from_rows(iter).expect("unable to build matrix from irregular iterator")
+    }
+}
+
 /// The matrix! macro allows the declaration of a Matrix from static data.
 /// All rows must have the same number of columns. The data will be copied
-/// into the matrix.
+/// into the matrix. There exist two forms:
+///
+/// - `matrix![(row1, row2, …, rowN)]`, each row being an array
+/// - `matrix![r1c1, r1c2, …, r1cN; r2c1, …,r2cN; …; rNc1, …, rNcN]`
 ///
 /// # Panics
 ///
@@ -522,27 +530,27 @@ impl<C> DerefMut for Matrix<C> {
 /// ```
 /// use pathfinding::matrix;
 ///
-/// let m = matrix![[10, 20, 30], [40, 50, 60]];
+/// let m1 = matrix![[10, 20, 30], [40, 50, 60]];
+/// assert_eq!(m1.columns, 3);
+/// assert_eq!(m1.rows, 2);
 ///
-/// assert_eq!(m.columns, 3);
-/// assert_eq!(m.rows, 2);
+/// let m2 = matrix![10, 20, 30; 40, 50, 60];
+/// assert_eq!(m1, m2);
 /// ```
 #[macro_export]
 macro_rules! matrix {
-    ($a:expr) => {{
+    () => { compile_error!("a matrix requires at least one row") };
+    ($a:expr $(, $b: expr)*$(,)?) => {{
         let mut m = pathfinding::matrix::Matrix::new_empty($a.len());
         m.extend(&$a).unwrap();
-        m
-    }};
-    ($a:expr, $($b: expr),+) => {{
-        let mut m = matrix!($a);
-        let mut r = 0;
         $(
-            m.extend(&$b).unwrap();
-        )+
+            m.extend(&$b).expect("all rows must have the same width");
+        )*
         m
     }};
-    ($a:expr, $($b: expr),+, ) => (matrix!($a, $($b),+))
+    ($($($a:expr),+$(,)?);+$(;)?) => {
+        matrix![$([$($a),+]),+]
+    };
 }
 
 /// Format error encountered while attempting to build a Matrix.
