@@ -280,6 +280,8 @@ fn matrix_macro() {
             i += 1;
         }
     }
+    let other_m = matrix![0, 1, 2; 3, 4, 5];
+    assert_eq!(m, other_m);
 }
 
 #[test]
@@ -288,6 +290,26 @@ fn matrix_macro_inconsistent_panic() {
     matrix![[0, 1, 2], [3, 4]];
 }
 
+#[test]
+#[should_panic]
+fn matrix_macro_inconsistent_panic_2() {
+    matrix![0, 1, 2; 3, 4];
+}
+
+#[test]
+fn macro_trailing_comma() {
+    // A trailing comma must be accepted
+    let m1 = matrix!([1, 2, 3], [4, 5, 6]);
+    let m2 = matrix!([1, 2, 3], [4, 5, 6],);
+    assert_eq!(m1, m2);
+    let m3 = matrix!(1, 2, 3; 4, 5, 6,);
+    assert_eq!(m1, m3);
+    let m4 = matrix!(1, 2, 3; 4, 5, 6;);
+    assert_eq!(m1, m4);
+    let m5 = matrix!(1, 2, 3; 4, 5, 6,;);
+    assert_eq!(m1, m5);
+}
+
 #[test]
 fn neighbours() {
     let m = matrix![[0, 1, 2], [3, 4, 5], [6, 7, 8]];
@@ -322,6 +344,24 @@ fn from_rows() {
     assert!(m.is_err());
 }
 
+#[test]
+fn from_iter() {
+    let m = (1..3)
+        .map(|n| (1..5).map(move |x| x * n))
+        .collect::<Matrix<_>>();
+    assert_eq!(m.rows, 2);
+    assert_eq!(m.columns, 4);
+    assert_eq!(m.to_vec(), vec![1, 2, 3, 4, 2, 4, 6, 8]);
+}
+
+#[test]
+#[should_panic]
+fn from_iter_error() {
+    let _ = (1..3)
+        .map(|n| (1..n).map(move |x| x * n))
+        .collect::<Matrix<_>>();
+}
+
 #[test]
 fn iter() {
     let m = matrix![[0, 1, 2], [3, 4, 5], [6, 7, 8]];
